define([
  "dojo/_base/declare",
  "dojo/_base/window",
  "dojo/dom-attr",
  "dojo/dom-style",
  "dijit/form/DateTextBox"
], function(declare, win, domAttr, domStyle, DateTextBox){
  return declare("wagby.DateTextBox", DateTextBox, {
    dateFormat: "yyyy-MM-dd",
    DEFAULT_DATE_FORMAT: "yyyy-MM-dd",
    cursorPosition: "left",   // left|right "left"

    postCreate: function(){
      this.inherited(arguments);
      this.setCursorPosition();
      this.setWidthByFormat();
    },

    setCursorPosition: function() {
      if (this.cursorPosition === "left") {
        return;
      }
      domStyle.set(this.textbox, "textAlign", this.cursorPosition);
    },

    _setDateFormatAttr: function(value) {
      this._set("dateFormat", value);
      this._set("pattern", this.getRegExpPattern(value));
    },

    setWidthByFormat: function() {
      var style = domAttr.get(this.domNode, "style");
      if (style && style.indexOf("width") !== -1) {
        return;
      }
      domStyle.set(this.domNode, "width", this.getWidthByDateFormat(this.dateFormat));
    },

    getWidthByDateFormat: function(format) {
      var width = format.length;
      if (width < this.DEFAULT_DATE_FORMAT.length) {
        width = this.DEFAULT_DATE_FORMAT.length;
      }
      if (format.lastIndexOf("ggge", 0) === 0 || format.lastIndexOf("ge", 0) === 0) {
        width = width + 1;
      }
      width = this.getWidthByTheme(format, width);
      return width + "em";
    },

    getWidthByTheme(format, formatLength) {
      var width = formatLength;
      var theme = domAttr.get(win.body(), "class");
      if (theme.lastIndexOf("flat_", 0) !== 0) {
        return width * 0.9;
      }
      if (format.indexOf("年") > 0) {
        width = width + 1;
      }
      return width;
    },

    getRegExpPattern: function(dateFormat) {
      // datePattern は利用せずに正規表現での入力チェックを行なう
      var pattern = dateFormatRegExpStr[dateFormat];
      if (!pattern) {
        // 日付フォーマットの正規表現文字列が取得できない場合は、
		// 入力チェックは行なわない
        return ".*";
      }
      return this.setAdditionalRegExpPattern(pattern);
    },

    setAdditionalRegExpPattern: function(pattern) {
      var regExpPattern = pattern + "|" + "\\d{1,8}" + "|" + this.getWarekiPattern();
      if (window.getMyDateRegExp) {
        regExpPattern = regExpPattern + "|" + window.getMyDateRegExp();
      }
      return regExpPattern;
    },

    getWarekiPattern: function(pattern) {
      var warekiPrefix = "";
      for (var i=0,l=wareki_shortcharpattern_a.length; i<l; i++) {
        warekiPrefix = warekiPrefix + wareki_shortcharpattern_a[i];
      }
      return "[" + warekiPrefix + "]\\d{1,2}\\s\\d{1,2}\\s\\d{1,2}";
    },

    format: function(/*Date*/ value, /*locale.__FormatOptions*/ constraints){
      if (!value) {
        return '';
      }
      return createDateObject(value, this.dateFormat);
    },

    "parse": function(value, constraints) {
      if (!value || value === "" || (value instanceof Date) && this._isInvalidDate(value)) {
        return value;
      }
      var dateObject;
      try {
        dateObject = createDateObject(value, this.dateFormat);
      } catch(e) {
      }
      if (!dateObject || (dateObject instanceof Date) && this._isInvalidDate(dateObject)) {
        return value;
      }
      return dateObject;
    },

    serialize: function(/*anything*/ val, /*Object?*/ options){
      if(val.toGregorian){
        val = val.toGregorian();
      }
      return _setDateWithFormat(val, this.dateFormat);
    },

    compare: function(/*Date*/ val1, /*Date*/ val2){
      var isInvalid1 = this._isInvalidDate(val1);
      var isInvalid2 = this._isInvalidDate(val2);
      if (isInvalid1 || isInvalid2){
        return (isInvalid1 && isInvalid2) ? 0 : (!isInvalid1 ? 1 : -1);
      }
      var fval1 = this.format(val1, this._unboundedConstraints);
      var fval2 = this.format(val2, this._unboundedConstraints);
      if (fval1 === fval2) {
        return 0;
      }
      var date1 = new Date(+fval1);
      var date2 = new Date(+(fval2 || new Date()));
      if(this._selector === "date" && this.dateFormat.indexOf("HH:mm:ss") === -1){
        date1.setHours(0, 0, 0, 0);
        date2.setHours(0, 0, 0, 0);
      } else if (this._selector === "time") {
        date1.setFullYear(0, 0, 0);
        date2.setFullYear(0, 0, 0);
      }
      if (date1 > date2) {
        return 1;
      }
      if (date1 < date2) {
        return -1;
      }
      return 0;
    },

    _setValueAttr: function(value, priorityChange, formattedValue){
      if (!value || value === "" || (value instanceof Date) && this._isInvalidDate(value)) {
        this.inherited(arguments, [null, priorityChange, formattedValue]);
        return;
      }
      if (this.value instanceof Date) {
        this.filterString = "";
      }
      var dateWithFormat = value;
      try {
        dateWithFormat = _setDateWithFormat(value, this.dateFormat);
      } catch(e) {
      }
      var dateObject;
      try {
        dateObject = createDateObject(value, this.dateFormat);
      } catch(e) {
      }
      if (!dateObject || (dateObject instanceof Date) && this._isInvalidDate(dateObject)) {
        this.inherited(arguments, [value, priorityChange, dateWithFormat]);
      } else {
        this.inherited(arguments, [dateObject, priorityChange, dateWithFormat]);
		if(this.value instanceof Date){
			this.filterString = "";
		}
        if (this.dropDown) {
          this.dropDown.set('value', dateObject, false);
          this._setDropDownDefaultValueAttr(dateObject);
        }
      }
    },

  });
});
